﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;

namespace CaptainQuery.CryptoHashProvider
{
    /// <summary>
    /// Wrapper around a SymmetricAlgorithm implementation
    /// which adds hash functionality 
    /// </summary>
    public class HashedCryptoServiceProvider : SymmetricAlgorithm
    {
        SymmetricAlgorithm crs = null;
        internal HashedCryptoServiceProvider(SymmetricAlgorithm crs)
        {
            this.crs = crs;
        }
        public override ICryptoTransform CreateDecryptor()
        {
            return new HashedCryptoTransform(false, crs.CreateDecryptor());
        }
        public override ICryptoTransform CreateDecryptor(byte[] rgbKey, byte[] rgbIV)
        {
            return new HashedCryptoTransform(false, crs.CreateDecryptor(rgbKey, rgbIV));         
        }
        public override ICryptoTransform CreateEncryptor()
        {
            return new HashedCryptoTransform(true, crs.CreateEncryptor());
            
        }
        public override ICryptoTransform CreateEncryptor(byte[] rgbKey, byte[] rgbIV)
        {
            return new HashedCryptoTransform(true, crs.CreateEncryptor(rgbKey, rgbIV));
        }
        public override void GenerateIV()
        {
            crs.GenerateIV();
            //return ret;
        }
        public override void GenerateKey()
        {
            crs.GenerateKey();            
        }
        public override KeySizes[] LegalBlockSizes
        {
            get
            {
                return crs.LegalBlockSizes;
                //return base.LegalBlockSizes;
            }
        }
        public override CipherMode Mode
        {
            get
            {
                return crs.Mode;

            }
            set
            {
                crs.Mode = value;

            }
        }
        public override int BlockSize
        {
            get
            {
                return crs.BlockSize;

            }
            set
            {
                crs.BlockSize = value;
            }
        }
        public override int FeedbackSize
        {
            get
            {
                return crs.FeedbackSize;

            }
            set
            {
                crs.FeedbackSize = value;

            }
        }
        public override byte[] IV
        {
            get
            {
                return crs.IV;

            }
            set
            {
                crs.IV = value;
            }
        }

        public override byte[] Key
        {
            get
            {
                return crs.Key;

            }
            set
            {
                crs.Key = value;
            }
        }
        public override KeySizes[] LegalKeySizes
        {
            get
            {
                return crs.LegalKeySizes;
            }
        }
        public override int KeySize
        {
            get
            {
                return crs.KeySize;
            }
            set
            {
                crs.KeySize = value;
            }
        }

        public override PaddingMode Padding
        {
            get
            {
                return crs.Padding;
            }
            set
            {
                crs.Padding = value;
            }
        }
    }
}
